package ch.bouquet.jpa;

import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.LockModeType;
import javax.persistence.NoResultException;
import javax.persistence.NonUniqueResultException;
import javax.persistence.OptimisticLockException;
import javax.persistence.Persistence;
import javax.persistence.Query;

import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;
import org.apache.log4j.helpers.Loader;

import ch.bouquet.jpa.data.DataLoader;
import ch.bouquet.jpa.domain.AuthUser;
import ch.bouquet.jpa.domain.Search;
import ch.bouquet.jpa.domain.Subscriber;
import ch.bouquet.jpa.domain.Search.Frequency;
import ch.bouquet.jpa.domain.Search.OutputFormat;
import ch.bouquet.jpa.domain.Search.State;
import ch.bouquet.jpa.persistence.AuthUserBean;
import ch.bouquet.jpa.persistence.SearchBean;
import ch.bouquet.jpa.persistence.SubscriberBean;
import ch.bouquet.jpa.util.DateHelper;

/**
 * <p>
 * CAS Enterprise Application Development mit Java EE (EADJ) - 2007<br/>
 * Modul: Data-Tier (JPA)<br/>
 * User admininstration (examination question)<br/>
 * Description:<br/>
 * This class start process the use cases for the examination example.
 * </p>
 * <p>Copyright (c) 2007 bouquet dot ch, 3032 Hinterkappelen. All rights reseved.</p>
 * @author    Marc Bouquet ( marc at bouquet dot ch, Swiss )
 * @version   1.0 - 25.05.2007
 */
public class JpaRunner
{
  /**
   * Default serial version uid
   */
  private static final long serialVersionUID = 1L;

  /**
   * Constant for the logging
   */
  private static Logger LOG = Logger.getLogger( JpaRunner.class );

  /**
   * Constant for start of log message
   */
  private static final String LOG_START = "START";

  /**
   * Constant for end of log message
   */
  private static final String LOG_END = "ENDE";

  /**
   * Constant for Persistence Name
   */
  private static String PERSISTENCE_NAME;

  //-----------------------------------------------------------------------------
  // C O N S T R U C T O R S
  //-----------------------------------------------------------------------------

  /**
   * Default constructor 
   */
  public JpaRunner()
  {
    //  initialize logging
    initLogging();
    LOG.info( "Start Logging ..." );
  }

  //-----------------------------------------------------------------------------
  // T E S T   S T A R T E R
  //-----------------------------------------------------------------------------

  /**
   * This method started the tester sequential
   * @param loops int number of runs
   * @throws Exception
   * @return boolean 
   */
  public boolean runSequentialTester( int loops ) throws Exception
  {
    // run use cases
    for ( int i = 0; i < loops; i++ )
    {
      usecase_1();
      usecase_2();
      usecase_3();
      usecase_4( false );
      usecase_5();
      usecase_6();
      usecase_7();
      usecase_8();
      usecase_9_1( false );
      usecase_9_2( false );
      if ( LOG.isInfoEnabled() )
      {
        int ii = i + 1;
        LOG.info( ii + ". Schlaufe sequenziell verarbeitet." );
      }
    }
    return true;
  }

  /**
   * This method started the tester parallel with threads
   * @param loops int number of runs
   * @throws Exception
   * @return boolean 
   */
  public boolean runParallelTester( int loops ) throws Exception
  {
    ExecutorService executor = Executors.newCachedThreadPool();
    // run use cases
    for ( int i = 0; i < loops; i++ )
    {
      usecase_1();
      usecase_2();
      usecase_3();
      usecase_4( false );
      usecase_5();
      usecase_6();
      usecase_7();
      usecase_8();

      Runnable r1 = new Runnable()
      {
        public void run()
        {
          usecase_9_1( false );
        }
      };

      Runnable r2 = new Runnable()
      {
        public void run()
        {
          usecase_9_2( false );
        }
      };
      executor.execute( r1 );
      executor.execute( r2 );
      if ( LOG.isInfoEnabled() )
      {
        int ii = i + 1;
        LOG.info( ii + ". Schlaufe wurde paralle verarbeitet." );
      }
    }
    executor.shutdown();
    return true;
  }

  //-----------------------------------------------------------------------------
  // U S E   C A S E S
  //-----------------------------------------------------------------------------

  /**
   * Anwendungsfall 1:<br/>
   * In diesem Anwendungsfall werden alle aktiven Benutzer ausgelesen. 
   * Ein Benutzer ist aktiv, wenn in der AuthUser-Tabelle das Attribut
   * <code>state</code> auf 1 gesetzt ist.
   */
  private void usecase_1()
  {
    EntityManagerFactory emf = Persistence.createEntityManagerFactory( PERSISTENCE_NAME );
    EntityManager em = emf.createEntityManager();
    try
    {
      if ( LOG.isInfoEnabled() )
      {
        LOG.info( getUseCaseLog( LOG_START, "1" ) );
      }
      long size = new AuthUserBean( em ).findSize( "SELECT COUNT(a) FROM AuthUser a WHERE a.state = 1" );
      if ( LOG.isInfoEnabled() )
      {
        LOG.info( "Es sind zurzeit <" + size + "> aktive Benutzer registriert. " );
      }
      if ( LOG.isInfoEnabled() )
      {
        LOG.info( getUseCaseLog( LOG_END, "1" ) );
      }
    }
    finally
    {
      em.close();
      emf.close();
    }
  }

  /**
   * Anwendungsfall 2:<br/>
   * In diesem Anwendungsfall werden von allen Benutzern 
   * der Primaerschluessel, der Login-Name sowie die zugewiesene 
   * Authentifizierungsrollen ausgelesen.
   */
  private void usecase_2()
  {
    EntityManagerFactory emf = Persistence.createEntityManagerFactory( PERSISTENCE_NAME );
    EntityManager em = emf.createEntityManager();
    try
    {
      if ( LOG.isInfoEnabled() )
      {
        LOG.info( getUseCaseLog( LOG_START, "2" ) );
      }
      List<AuthUser> list = new AuthUserBean( em ).findList( "SELECT a FROM AuthUser AS a INNER JOIN FETCH a.authRoleList" );
      if ( list != null && list.size() > 0 )
      {
        StringBuilder buf = new StringBuilder();
        Iterator<AuthUser> iter = list.iterator();
        while ( iter.hasNext() )
        {
          AuthUser user = (AuthUser) iter.next();
          buf.append( "Der Benutzer <" );
          buf.append( user.getAuthUserId() );
          buf.append( "> mit dem Login-Name <" );
          buf.append( user.getLogin() );
          buf.append( "> ist in der Rolle <" );
          buf.append( user.getAuthRoleList().iterator().next().getName() );
          buf.append( ">." );
          if ( iter.hasNext() )
          {
            buf.append( "\n" );
          }
        }
        if ( LOG.isInfoEnabled() )
        {
          LOG.info( buf.toString() );
        }
      }
      else
      {
        LOG.warn( "Es wurden keine Benutzer gefunden!" );
      }
      if ( LOG.isInfoEnabled() )
      {
        LOG.info( getUseCaseLog( LOG_END, "2" ) );
      }
    }
    finally
    {
      em.close();
      emf.close();
    }
  }

  /**
   * Anwendungsfall 3:<br/>
   * In diesem Anwendungsfall werden von jedem registrierten
   * Benutzer die Registrierungsdaten ausgelesen.
   */
  private void usecase_3()
  {
    EntityManagerFactory emf = Persistence.createEntityManagerFactory( PERSISTENCE_NAME );
    EntityManager em = emf.createEntityManager();
    try
    {
      if ( LOG.isInfoEnabled() )
      {
        LOG.info( getUseCaseLog( LOG_START, "3" ) );
      }
      List<Subscriber> list = new SubscriberBean( em ).findList( "SELECT s FROM Subscriber s" );
      if ( list != null && list.size() > 0 )
      {
        StringBuilder buf = new StringBuilder();
        Iterator<Subscriber> iter = list.iterator();
        while ( iter.hasNext() )
        {
          Subscriber subscriber = (Subscriber) iter.next();
          buf.append( subscriber.toStringFormatted() );
          if ( iter.hasNext() )
          {
            buf.append( "\n" );
          }
        }
        if ( LOG.isInfoEnabled() )
        {
          LOG.info( buf.toString() );
        }
      }
      else
      {
        LOG.warn( "Es wurden keine Registrierungsdaten gefunden!" );
      }
      if ( LOG.isInfoEnabled() )
      {
        LOG.info( getUseCaseLog( LOG_END, "3" ) );
      }
    }
    finally
    {
      em.close();
      emf.close();
    }
  }

  /**
   * Anwendungsfall 4:<br/>
   * In diesem Anwendungsfall werden zuerst die Registrierungsdaten des Benutzers 
   * mit der Email-Adresse <code>admin@bouquet.ch</code> ausgelesen und danach wird die Adresse, 
   * Postleitzahl und der Ort des Benutzers mutiert.
   * @param repeat boolean
   */
  private void usecase_4( boolean repeat )
  {
    EntityManagerFactory emf = Persistence.createEntityManagerFactory( PERSISTENCE_NAME );
    EntityManager em = emf.createEntityManager();
    try
    {
      if ( !repeat && LOG.isInfoEnabled() )
      {
        LOG.info( getUseCaseLog( LOG_START, "4" ) );
      }
      Subscriber subscriber = new SubscriberBean( em ).findSingle( "SELECT s FROM Subscriber s WHERE s.emailAddress = 'admin@bouquet.ch'" );
      if ( subscriber != null )
      {
        if ( LOG.isInfoEnabled() )
        {
          LOG.info( subscriber.toStringFormatted() );
        }
        try
        {
          Thread.sleep( (int) ( Math.random() * 200 ) );
        }
        catch ( Exception x )
        {
          // Don't do anything
        }
        em.getTransaction().begin();
        if ( PERSISTENCE_NAME.equals( "BFH4" ) )
        {
          em.lock( subscriber, LockModeType.WRITE );
        }
        subscriber.setAddress( "Stauffacherstrasse 65" );
        subscriber.setZip( "3003" );
        subscriber.setCity( "Bern" );
        em.merge( subscriber );
        em.flush();
        em.getTransaction().commit();
        if ( LOG.isInfoEnabled() )
        {
          LOG.info( "Der Datensatz wurde erfolgreich mutiert ...\n" + subscriber.toStringFormatted() );
        }
      }
      else
      {
        LOG.warn( "Es wurde kein Benutzer mit der Email-Adresse <admin@bouquet.ch> gefunden!" );
      }
      if ( LOG.isInfoEnabled() )
      {
        LOG.info( getUseCaseLog( LOG_END, "4" ) );
      }
    }
    catch ( NoResultException nrex )
    {
      LOG.warn( "Es wurde kein Eintrag gefunden mit dieser Abfrage!" );
      if ( LOG.isInfoEnabled() )
      {
        LOG.info( getUseCaseLog( LOG_END, "4" ) );
      }
    }
    catch ( OptimisticLockException olex )
    {
      em.getTransaction().rollback();
      if ( LOG.isInfoEnabled() )
      {
        LOG.info( "Der Datensatz wird zurzeit bearbeitet! Versuchen Sie es spaeter noch einmal ..." );
      }
      usecase_4( true );
    }
    catch ( NonUniqueResultException nuex )
    {
      LOG.error( "Es wurden mehr Resultate gefunden als erwartet!" );
      if ( LOG.isInfoEnabled() )
      {
        LOG.info( getUseCaseLog( LOG_END, "4" ) );
      }
    }
    finally
    {
      em.close();
      emf.close();
    }
  }

  /**
   * Anwendungsfall 5:<br/>
   * In diesem Anwendungsfall werden die von den Benutzern gespeicherten Suchen
   * analysiert. Es werden von der Search-Tabelle alle Suchabfragen mit der 
   * Frequenz <code>taeglich</code> ausgelesen und als Zusammenfassung ausgegeben.
   */
  private void usecase_5()
  {
    EntityManagerFactory emf = Persistence.createEntityManagerFactory( PERSISTENCE_NAME );
    EntityManager em = emf.createEntityManager();
    try
    {
      if ( LOG.isInfoEnabled() )
      {
        LOG.info( getUseCaseLog( LOG_START, "5" ) );
      }
      List<Search> list = new SearchBean( em ).findList( "SELECT s FROM Search s WHERE s.frequency='DAILY'" );
      if ( list != null && list.size() > 0 )
      {
        StringBuilder buf = new StringBuilder();
        buf.append( "Es wurden <" );
        buf.append( list.size() );
        buf.append( "> Suchabfragen gefunden, die <DAILY> durchgefuehrt werden." );
        buf.append( "\n\n" );
        Iterator<Search> iter = list.iterator();
        while ( iter.hasNext() )
        {
          Search search = (Search) iter.next();
          buf.append( search.toStringFormatted() );
          if ( iter.hasNext() )
          {
            buf.append( "\n\n" );
          }
        }
        if ( LOG.isInfoEnabled() )
        {
          LOG.info( buf.toString() );
        }
      }
      else
      {
        LOG.warn( "Es wurde keine taeglichen Suchabfragen gefunden!" );
      }
      if ( LOG.isInfoEnabled() )
      {
        LOG.info( getUseCaseLog( LOG_END, "5" ) );
      }
    }
    finally
    {
      em.close();
      emf.close();
    }
  }

  /**
   * Anwendungsfall 6:<br/>
   * In diesem Anwendungsfall wird dem Benutzer mit der Email-Adresse
   * <code>manager@bouquet.ch</code> eine woechentliche Suche hinzugefuegt.
   */
  private void usecase_6()
  {
    EntityManagerFactory emf = Persistence.createEntityManagerFactory( PERSISTENCE_NAME );
    EntityManager em = emf.createEntityManager();
    try
    {
      if ( LOG.isInfoEnabled() )
      {
        LOG.info( getUseCaseLog( LOG_START, "6" ) );
      }
      Search search = new Search();
      search.setSubscriber( new SubscriberBean( em ).findSingle( "SELECT s FROM Subscriber s WHERE s.emailAddress = 'manager@bouquet.ch'" ) );
      search.setName( "Java" );
      search.setCriteria( "Wortlaut der Marke=Java ist auch eine Insel" );
      search.setState( State.INACTIV );
      search.setFrequency( Frequency.WEEKLY );
      search.setOutputFormat( OutputFormat.PDF );
      search.setStartDat( null );

      // speichert die neue Suche in der Datenbank
      em.getTransaction().begin();
      search = new SearchBean( em ).save( search );
      em.getTransaction().commit();

      if ( LOG.isInfoEnabled() )
      {
        LOG.info( search.toStringFormatted() );
      }
      if ( LOG.isInfoEnabled() )
      {
        LOG.info( getUseCaseLog( LOG_END, "6" ) );
      }
    }
    catch ( NoResultException nrex )
    {
      LOG.warn( "Es wurde kein Eintrag gefunden mit dieser Abfrage!" );
      if ( LOG.isInfoEnabled() )
      {
        LOG.info( getUseCaseLog( LOG_END, "6" ) );
      }
    }
    catch ( NonUniqueResultException nuex )
    {
      LOG.error( "Es wurden mehr Resultate gefunden als erwartet!" );
      if ( LOG.isInfoEnabled() )
      {
        LOG.info( getUseCaseLog( LOG_END, "6" ) );
      }
    }
    catch ( Throwable e )
    {
      em.getTransaction().rollback();
      LOG.warn( "Throwable (rollback) is: <" + e.getMessage() + ">", e );
      if ( LOG.isInfoEnabled() )
      {
        LOG.info( getUseCaseLog( LOG_END, "6" ) );
      }
    }
    finally
    {
      em.close();
      emf.close();
    }
  }

  /**
   * Anwendungsfall 7:<br/>
   * In diesem Anwendungsfall werden die von den Benutzern gespeicherten Suchen
   * analysiert. Es werden von der Search-Tabelle alle Suchabfragen mit der 
   * Frequenz <code>woechentlich</code> ausgelesen und als Zusammenfassung ausgegeben.
   */
  private void usecase_7()
  {
    EntityManagerFactory emf = Persistence.createEntityManagerFactory( PERSISTENCE_NAME );
    EntityManager em = emf.createEntityManager();
    try
    {
      if ( LOG.isInfoEnabled() )
      {
        LOG.info( getUseCaseLog( LOG_START, "7" ) );
      }
      List<Search> list = new SearchBean( em ).findList( "SELECT s FROM Search s, IN(s.subscriber) sub WHERE s.frequency='WEEKLY' ORDER BY sub.subscriberId" );
      if ( list != null && list.size() > 0 )
      {
        StringBuilder buf = new StringBuilder();
        buf.append( "Es wurden <" );
        buf.append( list.size() );
        buf.append( "> Suchabfragen gefunden, die <WEEKLY> durchgefuehrt werden." );
        buf.append( "\n\n" );
        Iterator<Search> iter = list.iterator();
        while ( iter.hasNext() )
        {
          Search search = (Search) iter.next();
          buf.append( search.toStringFormatted() );
          if ( iter.hasNext() )
          {
            buf.append( "\n\n" );
          }
        }
        if ( LOG.isInfoEnabled() )
        {
          LOG.info( buf.toString() );
        }
      }
      else
      {
        LOG.warn( "Es wurde keine woechentlichen Suchabfragen gefunden!" );
      }
      if ( LOG.isInfoEnabled() )
      {
        LOG.info( getUseCaseLog( LOG_END, "7" ) );
      }
    }
    finally
    {
      em.close();
      emf.close();
    }
  }

  /**
   * Anwendungsfall 8:<br/>
   * In diesem Anwendungsfall soll der Benutzer mit der Email-Adresse
   * <code>user@bouquet.ch</code> geloescht werden.
   */
  private void usecase_8()
  {
    EntityManagerFactory emf = Persistence.createEntityManagerFactory( PERSISTENCE_NAME );
    EntityManager em = emf.createEntityManager();
    try
    {
      if ( LOG.isInfoEnabled() )
      {
        LOG.info( getUseCaseLog( LOG_START, "8" ) );
      }
      SubscriberBean subscriberBean = new SubscriberBean( em );
      em.getTransaction().begin();
      Subscriber subscriber = subscriberBean.findSingle( "SELECT s FROM Subscriber s WHERE s.emailAddress = 'user@bouquet.ch'" );
      if ( subscriberBean.remove( subscriber.getSubscriberId() ) )
      {
        if ( LOG.isInfoEnabled() )
        {
          LOG.info( "Die Daten des Benutzers <" + subscriber.getSubscriberId() + "> wurden geloescht!" );
        }
      }
      else
      {
        if ( LOG.isInfoEnabled() )
        {
          LOG.info( "Die Daten des Benutzers <" + subscriber.getSubscriberId() + "> konnten nicht geloescht werden!" );
        }
      }
      em.getTransaction().commit();
      if ( LOG.isInfoEnabled() )
      {
        LOG.info( getUseCaseLog( LOG_END, "8" ) );
      }
    }
    catch ( NoResultException nrex )
    {
      LOG.warn( "Es wurde kein Eintrag gefunden mit dieser Abfrage!" );
      if ( LOG.isInfoEnabled() )
      {
        LOG.info( getUseCaseLog( LOG_END, "8" ) );
      }
    }
    catch ( Throwable e )
    {
      em.getTransaction().rollback();
      LOG.warn( "Throwable (rollback) is: <" + e.getMessage() + ">", e );
    }
    finally
    {
      em.close();
      emf.close();
    }
  }

  /**
   * Anwendungsfall 9.1:<br/>
   * In diesem Anwendungsfall werden zwei Benutzer versuchen, denselben Datensatz, 
   * gleichzeitig zu mutieren.<br/>
   * Der Thread [1] liest den Benutzer mit der Email-Adresse
   * <code>admin@bouquet.ch</code>  aus und mutiert dessen Adressangaben wie folgt:
   * <ul>
   *  <li>Strasse <code>Kappelenring 49b</code></li>
   *  <li>PLZ <code>3032</code></li>
   *  <li>Ort <code>Hinterkappelen</code></li>
   * </ul>
   * @param repeat boolean
   */
  private void usecase_9_1( boolean repeat )
  {
    EntityManagerFactory emf = Persistence.createEntityManagerFactory( PERSISTENCE_NAME );
    EntityManager em = emf.createEntityManager();
    try
    {
      if ( !repeat && LOG.isInfoEnabled() )
      {
        LOG.info( getUseCaseLog( LOG_START, "9.1" ) );
      }
      Query query = em.createQuery( "SELECT s.subscriberId FROM Subscriber s WHERE s.emailAddress = 'admin@bouquet.ch'" );
      Subscriber subscriber = em.getReference( Subscriber.class, query.getSingleResult() );

      if ( subscriber != null )
      {
        if ( LOG.isInfoEnabled() )
        {
          LOG.info( subscriber.toStringFormatted() );
        }
        try
        {
          Thread.sleep( (int) ( Math.random() * 200 ) );
        }
        catch ( Exception x )
        {
          // Don't do anything
        }
        em.getTransaction().begin();
        if ( PERSISTENCE_NAME.equals( "BFH4" ) )
        {
          em.lock( subscriber, LockModeType.WRITE );
        }
        subscriber.setAddress( "Kappelenring 49b" );
        subscriber.setZip( "3032" );
        subscriber.setCity( "Hinterkappelen" );
        em.merge( subscriber );
        em.flush();
        em.getTransaction().commit();

        if ( LOG.isInfoEnabled() )
        {
          LOG.info( "Der Datensatz wurde erfolgreich mutiert ...\n" + subscriber.toStringFormatted() );
        }
      }
      else
      {
        LOG.warn( "Es wurde kein Benutzer mit der Email-Adresse <admin@bouquet.ch> gefunden!" );
      }
      if ( LOG.isInfoEnabled() )
      {
        LOG.info( getUseCaseLog( LOG_END, "9.1" ) );
      }
    }
    catch ( NoResultException nrex )
    {
      LOG.warn( "Es wurde kein Eintrag gefunden mit dieser Abfrage!" );
      if ( LOG.isInfoEnabled() )
      {
        LOG.info( getUseCaseLog( LOG_END, "9.1" ) );
      }
    }
    catch ( OptimisticLockException olex )
    {
      if ( PERSISTENCE_NAME.equals( "BFH4" ) )
      {
        em.getTransaction().rollback();
        LOG.warn( "Der Datensatz wird zurzeit bearbeitet! Versuchen Sie es spaeter noch einmal ...\n" );
        usecase_9_1( true );
      }
    }
    catch ( NonUniqueResultException nuex )
    {
      LOG.error( "Es wurden mehr Resultate gefunden als erwartet!" );
      if ( LOG.isInfoEnabled() )
      {
        LOG.info( getUseCaseLog( LOG_END, "9.1" ) );
      }
    }
    finally
    {
      em.close();
      emf.close();
    }
  }

  /**
   * Anwendungsfall 9.2:<br/>
   * In diesem Anwendungsfall werden zwei Benutzer versuchen, denselben Datensatz, 
   * gleichzeitig zu mutieren.<br/>
   * Der Thread [2] liest ebenfalls den Benutzer mit der Email-Adresse
   * <code>admin@bouquet.ch</code> aus und mutiert dessen Adressangaben wie folgt:
   * <ul>
   *  <li>Name <code>Boeser</code></li>
   *  <li>Vorname <code>Haecker</code></li>
   *  <li>Strasse <code>Hackeralle 33</code></li>
   *  <li>PLZ <code>8000</code></li>
   *  <li>Ort <code>Chur</code></li>
   * </ul>
   * @param repeat boolean
   */
  private void usecase_9_2( boolean repeat )
  {
    EntityManagerFactory emf = Persistence.createEntityManagerFactory( PERSISTENCE_NAME );
    EntityManager em = emf.createEntityManager();
    try
    {
      if ( !repeat && LOG.isInfoEnabled() )
      {
        LOG.info( getUseCaseLog( LOG_START, "9.2" ) );
      }
      Query query = em.createQuery( "SELECT s.subscriberId FROM Subscriber s WHERE s.emailAddress = 'admin@bouquet.ch'" );
      Subscriber subscriber = em.getReference( Subscriber.class, query.getSingleResult() );

      if ( subscriber != null )
      {
        if ( LOG.isInfoEnabled() )
        {
          LOG.info( subscriber.toStringFormatted() );
        }
        em.getTransaction().begin();
        if ( PERSISTENCE_NAME.equals( "BFH4" ) )
        {
          em.lock( subscriber, LockModeType.WRITE );
        }
        subscriber.setFirstName( "Boeser" );
        subscriber.setLastName( "Haecker" );
        subscriber.setAddress( "Hackeralle 33" );
        subscriber.setZip( "8000" );
        subscriber.setCity( "Chur" );
        em.merge( subscriber );
        em.flush();
        em.getTransaction().commit();

        if ( LOG.isInfoEnabled() )
        {
          LOG.info( "Der Datensatz wurde erfolgreich mutiert ...\n" + subscriber.toStringFormatted() );
        }
      }
      else
      {
        LOG.warn( "Es wurde kein Benutzer mit der Email-Adresse <admin@bouquet.ch> gefunden!" );
      }
      if ( LOG.isInfoEnabled() )
      {
        LOG.info( getUseCaseLog( LOG_END, "9.2" ) );
      }
    }
    catch ( NoResultException nrex )
    {
      LOG.warn( "Es wurde kein Eintrag gefunden mit dieser Abfrage!" );
      if ( LOG.isInfoEnabled() )
      {
        LOG.info( getUseCaseLog( LOG_END, "9.2" ) );
      }
    }
    catch ( OptimisticLockException olex )
    {
      if ( PERSISTENCE_NAME.equals( "BFH4" ) )
      {
        em.getTransaction().rollback();
        LOG.warn( "Der Datensatz wird zurzeit bearbeitet! Versuchen Sie es spaeter noch einmal ...\n" );
        usecase_9_2( true );
      }
    }
    catch ( NonUniqueResultException nuex )
    {
      LOG.error( "Es wurden mehr Resultate gefunden als erwartet!" );
      if ( LOG.isInfoEnabled() )
      {
        LOG.info( getUseCaseLog( LOG_END, "9.2" ) );
      }
    }
    finally
    {
      em.close();
      emf.close();
    }
  }

  /**
   * This method clear the database data.
   */
  private void clearDatabase()
  {
    EntityManagerFactory emf = Persistence.createEntityManagerFactory( PERSISTENCE_NAME );
    EntityManager em = emf.createEntityManager();
    try
    {
      Query query = em.createQuery( "SELECT s.subscriberId FROM Subscriber s" );
      List<Integer> results = query.getResultList();
      
      
      em.getTransaction().begin();
      for ( int p: results )
      {
        Subscriber subscriber = em.find( Subscriber.class, p );
        if ( subscriber != null )
        {
          // remove subscriber
          em.remove( subscriber );
          em.flush();
        }
      }
      em.getTransaction().commit();
    }
    catch ( NoResultException nrex )
    {
      LOG.warn( "Es wurde kein Eintrag gefunden mit dieser Abfrage!" );
    }
    finally
    {
      em.close();
      emf.close();
    }
  }

  //-----------------------------------------------------------------------------
  // G E T T E R - / S E T T E R - M E T H O D S
  //-----------------------------------------------------------------------------

  //-----------------------------------------------------------------------------
  // H E L P E R - M E T H O D S
  //-----------------------------------------------------------------------------

  /**
   * Initialize logging.
   */
  private static void initLogging()
  {
    PropertyConfigurator.configure( Loader.getResource( "log4j.properties" ) );
    PropertyConfigurator.configureAndWatch( "log4j.properties" );
  }

  /**
   * Return the usecase log message.
   * @param value String start or end
   * @param number String number of the usecase
   * @return String formatted usecase log message
   */
  private String getUseCaseLog( String value, String number )
  {
    StringBuilder buf = new StringBuilder();
    buf.append( "#=========================================================================\n" );
    buf.append( "# " );
    buf.append( value );
    buf.append( " DES ANWENDUNGSFALL [ " );
    buf.append( number );
    buf.append( " ]" );
    buf.append( "\n#=========================================================================" );
    return buf.toString();
  }

  /**
   * Return the project start information
   * @param startDate date of the start
   * @return information
   */
  private static String getInformation( Date startDate )
  {
    final StringBuilder buf = new StringBuilder();
    buf.append( "#=== START DES TEST-PROZESSES =============================================" );
    buf.append( "\n" );
    buf.append( "# CAS Enterprise Application Development mit Java EE (EADJ) - 2007" );
    buf.append( "\n" );
    buf.append( "# Modul: Data-Tier (JPA)" );
    buf.append( "\n" );
    buf.append( "# Uebung: Benutzerverwaltung (Pruefungsaufgabe)" );
    buf.append( "\n" );
    buf.append( "# Copyright (c) 2007 bouquet dot ch, 3032 Hinterkappelen. All rights reseved." );
    buf.append( "\n" );
    buf.append( "# Marc Bouquet ( marc at bouquet dot ch, Swiss )" );
    buf.append( "\n" );
    buf.append( "# 1.0 - 18.06.2007" );
    buf.append( "\n#==========================================================================\n" );
    buf.append( "# Isolations-Level ist: <" );
    buf.append( PERSISTENCE_NAME.substring( 3, 4 ) );
    buf.append( ">\n# Gestartet um: " );
    buf.append( DateHelper.toString( startDate ) );
    buf.append( "\n#==========================================================================" );
    return buf.toString();
  }

  //-----------------------------------------------------------------------------
  // M A I N - M E T H O D
  //-----------------------------------------------------------------------------

  /**
   * Start the test application for java persistence API
   * @param args no arguments required
   */
  public static void main( String[] args )
  {
    Date startDate = new Date( System.currentTimeMillis() );

    try
    {
      // initialize logging
      initLogging();
      if ( args.length != 3 )
      {
        System.out.println( "usage: Tester <help>" );
        LOG.error( "usage: Tester <help>" );
        System.exit( 1 );
      }

      // check isolation level
      String isolation = args[0];
      if ( isolation != null && ( isolation.equalsIgnoreCase( "0" ) || isolation.equalsIgnoreCase( "2" ) || isolation.equalsIgnoreCase( "4" ) ) )
      {
        PERSISTENCE_NAME = "BFH" + isolation;
      }
      else
      {
        System.out.println( "usage: Tester <help> for isolation level" );
        LOG.error( "usage: Tester <help> for isolation level" );
        System.exit( 1 );
      }

      if ( LOG.isInfoEnabled() )
      {
        LOG.info( getInformation( startDate ) );
      }

      // load database data
      if ( LOG.isInfoEnabled() )
      {
        LOG.info( "--- START LADEN DER DATENBANK --------------------------------------------" );
      }
      new DataLoader().load( PERSISTENCE_NAME );
      if ( LOG.isInfoEnabled() )
      {
        LOG.info( "--- ENDE DES LADEN DER DATENBANK -----------------------------------------" );
      }

      JpaRunner run = new JpaRunner();
      boolean isTerminated = false;

      // check size of loops
      int loops = Integer.parseInt( args[1] );
      if ( loops == 0 || loops == -1 )
      {
        loops = 1;
      }

      // parallel tester with threads
      if ( args[2].equalsIgnoreCase( "p" ) )
      {
        isTerminated = run.runParallelTester( loops );
      }
      // sequential tester with number of runs
      else if ( args[2].equalsIgnoreCase( "s" ) )
      {
        isTerminated = run.runSequentialTester( loops );
      }
      else
      {
        System.out.println( "usage: Tester <help> for process types" );
        LOG.error( "usage: Tester <help> for process types" );
        System.exit( 1 );
      }

      if ( isTerminated )
      {
        run.clearDatabase();
        // clear the database data
        if ( LOG.isInfoEnabled() )
        {
          LOG.info( "Testdaten wurden aus der Datenbank geloescht ..." );
        }
      }
    }
    catch ( Exception ex )
    {
      LOG.error( "Exception: " + ex.getMessage(), ex );
    }
    finally
    {
      Date endDate = new Date( System.currentTimeMillis() );
      if ( LOG.isInfoEnabled() )
      {
        StringBuilder buf = new StringBuilder();
        buf.append( "=== ENDE DES TEST-PROZESSES ==============================================" );
        buf.append( "\n" );
        buf.append( "Total Zeit: " );
        buf.append( ( endDate.getTime() - startDate.getTime() ) / 1000 );
        buf.append( " Sekunden\nBeendet um: " );
        buf.append( DateHelper.toString( endDate ) );
        buf.append( "\n==========================================================================" );
        LOG.info( buf.toString() );
      }
    }
  }
}
