/** Copyright (c) 2007 bouquet dot ch, 3032 hinterkappelen. All rights reseved. 
 * Alle Rechte vorbehalten. Reproduktion zu nicht kommerziellem Gebrauch mit Quellenangabe 
 * gestattet. Reproduktion (auch auszugsweise) zum kommerziellen Gebrauch für Vortragszwecke 
 * sind nur mit schriftlicher Bewilligung des Verfassers gestattet.
 */
package ch.bouquet.jpa.data;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import java.util.TreeSet;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;

import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;
import org.apache.log4j.helpers.Loader;

import ch.bouquet.jpa.domain.AuthRole;
import ch.bouquet.jpa.domain.AuthUser;
import ch.bouquet.jpa.domain.Search;
import ch.bouquet.jpa.domain.Subscriber;
import ch.bouquet.jpa.domain.Search.Frequency;
import ch.bouquet.jpa.domain.Search.OutputFormat;
import ch.bouquet.jpa.domain.Search.State;
import ch.bouquet.jpa.persistence.SearchBean;
import ch.bouquet.jpa.persistence.SubscriberBean;
import ch.bouquet.jpa.util.DateHelper;

/**
 * <p>
 * CAS Enterprise Application Development mit Java EE (EADJ) - 2007<br/>
 * Modul: Data-Tier (JPA)<br/>
 * User admininstration (examination question)<br/>
 * Description:<br/>
 * Helper class for load test data into the database.
 * </p>
 * <p>Copyright (c) 2007 bouquet dot ch, 3032 Hinterkappelen. All rights reseved.</p>
 * @author    Marc Bouquet ( marc at bouquet dot ch, Swiss )
 * @version   1.0 - 25.05.2007
 */
public class DataLoader
{
  /**
   * Default serial version uid
   */
  private static final long serialVersionUID = 1L;

  /**
   * constant for the logging
   */
  private final Logger LOG = Logger.getLogger( DataLoader.class );

  /**
   * list of loaded authenticated roles including the auto generated primary keys
   */
  List<AuthRole> authRoleList = new ArrayList<AuthRole>();

  /**
   * list of loaded subscribers including the auto generated primary keys and transient fields
   */
  List<Subscriber> subscriberList = new ArrayList<Subscriber>();

  /**
   * list of loaded searchs including the auto generated primary keys and transient fields
   */
  List<Search> searchList = new ArrayList<Search>();

  /**
   * Default constructor.
   */
  public DataLoader()
  {
    initLogging();
    // BasicConfigurator.configure();
  }

  /**
   * Initialize logging.
   */
  private void initLogging()
  {
    PropertyConfigurator.configure( Loader.getResource( "log4j.properties" ) );
    PropertyConfigurator.configureAndWatch( "log4j.properties" );
  }

  /**
   * Load testdata into the eadj database
   * @param ctx String context
   * @throws Exception
   */
  public void load( String ctx ) throws Exception
  {
    EntityManagerFactory emf = Persistence.createEntityManagerFactory( ctx );
    EntityManager em = emf.createEntityManager();

    try
    {
      // load authentication roles
      for ( Iterator iter = createAuthRoles().iterator(); iter.hasNext(); )
      {
        authRoleList.add( (AuthRole) iter.next() );
      }

      // load subscribers
      for ( Iterator iter = createSubscripers().iterator(); iter.hasNext(); )
      {
        em.getTransaction().begin();
        Subscriber subscriber = new SubscriberBean( em ).save( (Subscriber) iter.next() );
        em.getTransaction().commit();
        if ( LOG.isDebugEnabled() )
        {
          LOG.debug( subscriber.toString() );
        }
        subscriberList.add( subscriber );
      }

      // load searchs
      for ( Iterator iter = createSearchs().iterator(); iter.hasNext(); )
      {
        em.getTransaction().begin();
        Search search = new SearchBean( em ).save( (Search) iter.next() );
        em.getTransaction().commit();
        if ( LOG.isDebugEnabled() )
        {
          LOG.debug( search.toString() );
        }
        searchList.add( search );
      }
    }
    finally
    {
      em.close();
      emf.close();
    }
  }

  /**
   * Create test data for the subscribers
   * @return List<Subscriber>
   */
  private List<Subscriber> createSubscripers()
  {
    List<Subscriber> list = new ArrayList<Subscriber>();
    list.add( createSubscriberAdmin() );
    list.add( createSubscriberManager() );
    list.add( createSubscriberUser() );
    list.add( createSubscriberGuest() );
    return list;
  }

  /**
   * Create a subscriber for the admin user
   * @return Subscriber
   */
  private Subscriber createSubscriberAdmin()
  {
    Subscriber sub = new Subscriber();
    sub.setAuthUser( createAuthUser( "admin@bouquet.ch", "admin", "SHA-1", 1, authRoleList.get( 0 ) ) );
    sub.setFirstName( "Marc" );
    sub.setLastName( "Bouquet" );
    sub.setEmailAddress( "admin@bouquet.ch" );
    sub.setAddress( "Kappelenring 49b" );
    sub.setZip( "3032" );
    sub.setCity( "Hinterkappelen" );
    sub.setCountry( "CH" );
    return sub;
  }

  /**
   * Create a subscriber for the manager user
   * @return Subscriber
   */
  private Subscriber createSubscriberManager()
  {
    Subscriber sub = new Subscriber();
    sub.setAuthUser( createAuthUser( "manager@bouquet.ch", "manager", "SHA-1", 1, authRoleList.get( 1 ) ) );
    sub.setFirstName( "Thomas" );
    sub.setLastName( "Iten" );
    sub.setEmailAddress( "manager@bouquet.ch" );
    sub.setAddress( "Friedlistrasse 14" );
    sub.setZip( "3006" );
    sub.setCity( "Bern" );
    sub.setCountry( "CH" );
    return sub;
  }

  /**
   * Create a subscriber for the user user
   * @return Subscriber
   */
  private Subscriber createSubscriberUser()
  {
    Subscriber sub = new Subscriber();
    sub.setAuthUser( createAuthUser( "user@bouquet.ch", "user", "SHA-1", 1, authRoleList.get( 2 ) ) );
    sub.setFirstName( "Hans" );
    sub.setLastName( "Muster" );
    sub.setEmailAddress( "user@bouquet.ch" );
    sub.setAddress( "Musterweg 23" );
    sub.setZip( "7000" );
    sub.setCity( "Chur" );
    sub.setCountry( "CH" );
    return sub;
  }

  /**
   * Create a subscriber for the guest user
   * @return Subscriber
   */
  private Subscriber createSubscriberGuest()
  {
    Subscriber sub = new Subscriber();
    sub.setAuthUser( createAuthUser( "guest@bouquet.ch", "guest", "SHA-1", 0, authRoleList.get( 3 ) ) );
    sub.setFirstName( "Greg" );
    sub.setLastName( "Miles" );
    sub.setEmailAddress( "guest@bouquet.ch" );
    sub.setAddress( "14 Hazelwood Lane" );
    sub.setZip( "Co. Down,BT23 6DG" );
    sub.setCity( "Comber" );
    sub.setCountry( "UK" );
    return sub;
  }

  /**
   * Create test data for the authenticated roles
   * @return List<AuthRole>
   */
  private List<AuthRole> createAuthRoles()
  {
    List<AuthRole> list = new ArrayList<AuthRole>();
    list.add( createAuthRole( "admin" ) );
    list.add( createAuthRole( "manager" ) );
    list.add( createAuthRole( "user" ) );
    list.add( createAuthRole( "guest" ) );
    return list;
  }

  /**
   * Create a authenticated role
   * @param name of the role
   * @return AuthRole
   */
  private AuthRole createAuthRole( String name )
  {
    return ( new AuthRole( name ) );
  }

  /**
   * Create a authenticated user
   * @param uid String
   * @param pwd String
   * @param encryptionVersion String
   * @param state int
   * @param role AuthRole
   * @return AuthUser
   */
  private AuthUser createAuthUser( String uid, String pwd, String encryptionVersion, int state, AuthRole role )
  {
    Set<AuthRole> list = new TreeSet<AuthRole>();
    list.add( role );
    return ( new AuthUser( uid, pwd, encryptionVersion, state, list ) );
  }

  /**
   * Create test data for the searchs
   * @return List<Search>
   */
  private List<Search> createSearchs()
  {
    List<Search> list = new ArrayList<Search>();
    list.add( createSearchCola() );
    list.add( createSearchPepsi() );
    list.add( createSearchHeineke() );
    list.add( createSearchCarlsberg() );
    list.add( createSearchCaffeLatte() );
    return list;
  }

  /**
   * Create a search for the admin
   * @return Search
   */
  private Search createSearchCola()
  {
    Search search = new Search();
    search.setSubscriber( subscriberList.get( 0 ) );
    search.setName( "Cola-PDF" );
    search.setCriteria( "Wortlaut der Marke=Cola" );
    search.setState( State.INACTIV );
    search.setFrequency( Frequency.DAILY );
    search.setOutputFormat( OutputFormat.PDF );
    search.setStartDat( null );
    return search;
  }

  /**
   * Create a search for the admin
   * @return Search
   */
  private Search createSearchPepsi()
  {
    Search search = new Search();
    search.setSubscriber( subscriberList.get( 0 ) );
    search.setName( "Pepsi-XML" );
    search.setCriteria( "Wortlaut der Marke=Pepsi" );
    search.setState( State.INACTIV );
    search.setFrequency( Frequency.WEEKLY );
    search.setOutputFormat( OutputFormat.XML );
    search.setStartDat( null );
    return search;
  }

  /**
   * Create a search for the manager
   * @return Search
   */
  private Search createSearchHeineke()
  {
    Search search = new Search();
    search.setSubscriber( subscriberList.get( 1 ) );
    search.setName( "Hein eke" );
    search.setCriteria( "Wortlaut der Marke=Heineke" );
    search.setState( State.ACTIV );
    search.setFrequency( Frequency.MONTHLY );
    search.setOutputFormat( OutputFormat.XML );
    search.setStartDat( DateHelper.toTimestamp( "18.06.2007" ) );
    return search;
  }

  /**
   * Create a search for the manager
   * @return Search
   */
  private Search createSearchCarlsberg()
  {
    Search search = new Search();
    search.setSubscriber( subscriberList.get( 1 ) );
    search.setName( "Carlsberg.xls" );
    search.setCriteria( "Wortlaut der Marke=Carlsberg" );
    search.setState( State.ACTIV );
    search.setFrequency( Frequency.DAILY );
    search.setOutputFormat( OutputFormat.EXCEL );
    search.setStartDat( DateHelper.toTimestamp( "02.09.2007" ) );
    return search;
  }

  /**
   * Create a search for the user
   * @return Search
   */
  private Search createSearchCaffeLatte()
  {
    Search search = new Search();
    search.setSubscriber( subscriberList.get( 2 ) );
    search.setName( "emmi pdf" );
    search.setCriteria( "Wortlaut der Marke=Caffe Latte" );
    search.setState( State.ACTIV );
    search.setFrequency( Frequency.WEEKLY );
    search.setOutputFormat( OutputFormat.PDF );
    search.setStartDat( DateHelper.toTimestamp( "26.11.2007" ) );
    return search;
  }

  /**
   * Load test data into eadj database
   * @param args no arguments required
   */
  public static void main( String[] args )
  {
    try
    {
      new DataLoader().load( "BFH0" );
    }
    catch ( Exception e )
    {
      e.printStackTrace();
    }
  }
}