/** Copyright (c) 2007 bouquet dot ch, 3032 hinterkappelen. All rights reseved. 
 * Alle Rechte vorbehalten. Reproduktion zu nicht kommerziellem Gebrauch mit Quellenangabe 
 * gestattet. Reproduktion (auch auszugsweise) zum kommerziellen Gebrauch für Vortragszwecke 
 * sind nur mit schriftlicher Bewilligung des Verfassers gestattet.
 */
package ch.bouquet.jpa.domain;

import java.io.Serializable;
import java.util.Date;
import java.util.Iterator;
import java.util.Set;
import java.util.TreeSet;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

import org.apache.log4j.Logger;

/**
 * <p>
 * CAS Enterprise Application Development mit Java EE (EADJ) - 2007<br/>
 * Modul: Data-Tier (JPA)<br/>
 * User admininstration (examination question)<br/>
 * Description:<br/>
 * Authenticated user business object implemented as plain old java object.
 * </p>
 * <p>Copyright (c) 2007 bouquet dot ch, 3032 Hinterkappelen. All rights reseved.</p>
 * @author    Marc Bouquet ( marc at bouquet dot ch, Swiss )
 * @version   1.0 - 25.05.2007
 */
@Entity
@Table( name = "AUTH_USER" )
public class AuthUser implements Serializable
{
  /**
   * constant for the logging
   */
  @SuppressWarnings( "unused" )
  private static Logger LOG = Logger.getLogger( AuthUser.class );

  /**
   * Default serial version uid
   */
  private static final long serialVersionUID = 1L;

  /**
   * auth user id (primary key)
   */
  @Id
  @GeneratedValue
  @Column( name = "AUTH_USER_ID" )
  private int authUserId;

  /**
   * login
   */
  @Column( name = "LOGIN" )
  private String login;

  /**
   * password
   */
  @Column( name = "PASSWORD" )
  private String password;

  /**
   * encryption version
   */
  @Column( name = "ENCRYPTION_VERSION" )
  private String encryptionVersion;

  /**
   * state
   */
  @Column( name = "STATE" )
  private int state;

  /**
   * last login date
   */
  @Column( name = "LAST_LOGIN_DAT" )
  @Temporal( TemporalType.DATE )
  private Date lastLoginDat;

  /**
   * Authenticated role list of the authenticated user.<br />
   * Notes:<br /> 
   * - Unidirectional one to many relationship.<br /> 
   * - The mapping will be done with the join table auth_user_role.
   */
  @OneToMany( cascade = CascadeType.ALL, fetch = FetchType.LAZY )
  @JoinTable( name = "AUTH_USER_ROLE", joinColumns = { @JoinColumn( name = "AUTH_USER_ID" ) }, inverseJoinColumns = { @JoinColumn( name = "AUTH_ROLE_ID" ) } )
  private Set<AuthRole> authRoleList = new TreeSet<AuthRole>();

  //-----------------------------------------------------------------------------
  // C O N S T R U C T O R S
  //-----------------------------------------------------------------------------

  /**
   * Default constructor
   */
  public AuthUser()
  {
    // Don't do anything
  }

  /**
   * Constructor
   * @param uid String
   * @param pwd String
   * @param encryptionVersion String
   * @param state int
   * @param authRoleList Set
   */
  public AuthUser( String uid, String pwd, String encryptionVersion, int state, Set<AuthRole> authRoleList )
  {
    setLogin( uid );
    setPassword( pwd );
    setEncryptionVersion( encryptionVersion );
    setState( state );
    setAuthRoleList( authRoleList );
  }

  /**
   * Constructor
   * @param uid String
   * @param pwd String
   * @param encryptionVersion String
   * @param state int
   * @param lastLoginDat Date
   * @param authRoleList Set
   */
  public AuthUser( String uid, String pwd, String encryptionVersion, int state, Date lastLoginDat, Set<AuthRole> authRoleList )
  {
    setLogin( uid );
    setPassword( pwd );
    setEncryptionVersion( encryptionVersion );
    setState( state );
    setLastLoginDat( lastLoginDat );
    setAuthRoleList( authRoleList );
  }

  //---------------------------------------------------------------------------
  // toString() method
  //---------------------------------------------------------------------------

  /**
   * Return all attributes as String
   * @return Object as String
   */
  @Override
  public String toString()
  {
    StringBuilder buf = new StringBuilder();
    buf.append( " --- toString() of ch.bouquet.jpa.domain.AuthUser ------------------------\n" );
    buf.append( "  authUserId             <" );
    buf.append( getAuthUserId() );
    buf.append( ">\n" );
    buf.append( "  login                  <" );
    buf.append( getLogin() );
    buf.append( ">\n" );
    buf.append( "  password               <" );
    buf.append( getPassword() );
    buf.append( ">\n" );
    buf.append( "  encryption version     <" );
    buf.append( getEncryptionVersion() );
    buf.append( ">\n" );
    buf.append( "  state                  <" );
    buf.append( getState() );
    buf.append( ">\n" );
    buf.append( "  last login date        <" );
    buf.append( getLastLoginDat() );
    buf.append( ">\n" );

    // auth role fields
    if ( authRoleList.size() == 0 )
    {
      buf.append( "\n  auth role list is      <null>" );
    }
    else
    {
      int count = 1;

      for ( Iterator iter = authRoleList.iterator(); iter.hasNext(); )
      {
        AuthRole role = (AuthRole) iter.next();
        buf.append( "\n  auth role element      <" );
        buf.append( count );
        buf.append( ">\n" );
        buf.append( role.toString() );
        count++;
      }
    }
    return buf.toString();
  }

  //-----------------------------------------------------------------------------
  // G E T T E R - / S E T T E R - M E T H O D S
  //-----------------------------------------------------------------------------

  /**
   * Gets the authUserId.
   * @return the authUserId
   */
  public int getAuthUserId()
  {
    return this.authUserId;
  }

  /**
   * Sets the authUserId.
   * @param authUserId The authUserId to set
   */
  public void setAuthUserId( int authUserId )
  {
    this.authUserId = authUserId;
  }

  /**
   * Gets the login.
   * @return the login
   */
  public String getLogin()
  {
    return this.login;
  }

  /**
   * Sets the login.
   * @param login The login to set
   */
  public void setLogin( String login )
  {
    this.login = login;
  }

  /**
   * Gets the password.
   * @return the password
   */
  public String getPassword()
  {
    return this.password;
  }

  /**
   * Sets the password.
   * @param password The password to set
   */
  public void setPassword( String password )
  {
    this.password = password;
  }

  /**
   * Gets the encryptionVersion.
   * @return the encryptionVersion
   */
  public String getEncryptionVersion()
  {
    return this.encryptionVersion;
  }

  /**
   * Sets the encryptionVersion.
   * @param encryptionVersion The encryptionVersion to set
   */
  public void setEncryptionVersion( String encryptionVersion )
  {
    this.encryptionVersion = encryptionVersion;
  }

  /**
   * Gets the state.
   * @return the state
   */
  public int getState()
  {
    return this.state;
  }

  /**
   * Sets the state.
   * @param state The state to set
   */
  public void setState( int state )
  {
    this.state = state;
  }

  /**
   * Gets the lastLoginDat.
   * @return the lastLoginDat
   */
  public Date getLastLoginDat()
  {
    return this.lastLoginDat;
  }

  /**
   * Sets the lastLoginDat.
   * @param lastLoginDat The lastLoginDat to set
   */
  public void setLastLoginDat( Date lastLoginDat )
  {
    this.lastLoginDat = lastLoginDat;
  }

  /**
   * Get auth role list
   * @return authRoleList
   */
  public Set<AuthRole> getAuthRoleList()
  {
    return authRoleList;
  }

  /**
   * Set auth role list
   * @param authRoleList
   */
  public void setAuthRoleList( Set<AuthRole> authRoleList )
  {
    this.authRoleList = authRoleList;
  }
}
