/** Copyright (c) 2007 bouquet dot ch, 3032 hinterkappelen. All rights reseved. 
 * Alle Rechte vorbehalten. Reproduktion zu nicht kommerziellem Gebrauch mit Quellenangabe 
 * gestattet. Reproduktion (auch auszugsweise) zum kommerziellen Gebrauch für Vortragszwecke 
 * sind nur mit schriftlicher Bewilligung des Verfassers gestattet.
 */
package ch.bouquet.jpa.domain;

import java.io.Serializable;
import java.util.Date;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

import org.apache.log4j.Logger;

/**
 * <p>
 * CAS Enterprise Application Development mit Java EE (EADJ) - 2007<br/>
 * Modul: Data-Tier (JPA)<br/>
 * User admininstration (examination question)<br/>
 * Description:<br/>
 * Search business object implemented as plain old java object.
 * </p>
 * <p>Copyright (c) 2007 bouquet dot ch, 3032 Hinterkappelen. All rights reseved.</p>
 * @author    Marc Bouquet ( marc at bouquet dot ch, Swiss )
 * @version   1.0 - 25.05.2007
 */
@Entity
@Table( name = "SEARCH" )
public class Search implements Serializable
{
  /**
   * constant for the logging
   */
  @SuppressWarnings( "unused" )
  private static Logger LOG = Logger.getLogger( Search.class );

  /**
   * Default serial version uid
   */
  private static final long serialVersionUID = 1L;

  /**
   * Enumeration
   */
  public enum OutputFormat
  {
    /**
     * Das Ausgabeformat ist eine PDF-Datei
     */
    PDF,
    /**
     * Das Ausgabeformat ist eine XML-Datei
     */
    XML,
    /**
     * Das Ausgabeformat ist eine EXCEL-Datei
     */
    EXCEL
  }

  /**
   * Enumeration
   */
  public enum Frequency
  {
    /**
     * Die Suche wird täglich ausgeführt
     */
    DAILY,
    /**
     * Die Suche wird wöchentlich ausgeführt
     */
    WEEKLY,
    /**
     * Die Suche wird monatlich ausgeführt
     */
    MONTHLY
  }

  /**
   * Enumeration
   */
  public enum State
  {
    /**
     * search is activ
     */
    ACTIV,

    /**
     * search is inactive
     */
    INACTIV
  }

  /**
   * search id (primary key)
   */
  @Id
  @GeneratedValue
  @Column( name = "SEARCH_ID" )
  private int searchId;

  /**
   * subscriper id (foreign key)
   */
  @ManyToOne( cascade = CascadeType.MERGE, fetch = FetchType.EAGER )
  @JoinColumn( name = "SUBSCRIBER_ID" )
  private Subscriber subscriber;

  /**
   * search name
   */
  @Column( name = "NAME" )
  private String name;

  /**
   * search criteria
   */
  @Column( name = "CRITERIA" )
  private String criteria;

  /**
   * search state
   */
  @Column( name = "STATE" )
  @Enumerated( EnumType.STRING )
  private State state;

  /**
   * frequency
   */
  @Column( name = "FREQUENCY" )
  @Enumerated( EnumType.STRING )
  private Frequency frequency;

  /**
   * output format
   */
  @Column( name = "OUTPUT_FORMAT" )
  @Enumerated( EnumType.STRING )
  private OutputFormat outputFormat;

  /**
   * start date
   */
  @Column( name = "START_DAT" )
  @Temporal( TemporalType.DATE )
  private Date startDat;

  //-----------------------------------------------------------------------------
  // C O N S T R U C T O R S
  //-----------------------------------------------------------------------------

  /**
   * Default constructor
   */
  public Search()
  {
    // Don't do anything
  }

  /**
   * Constructor
   * @param subscriber Subscriber
   * @param name String
   * @param criteria String
   * @param state State
   * @param frequency Frequency
   * @param outputFormat OutputFormat
   * @param startDate Date
   */
  public Search( Subscriber subscriber, String name, String criteria, State state, Frequency frequency, OutputFormat outputFormat, Date startDate )
  {
    setSubscriber( subscriber );
    setName( name );
    setCriteria( criteria );
    setState( state );
    setFrequency( frequency );
    setOutputFormat( outputFormat );
    setStartDat( startDate );
  }

  //---------------------------------------------------------------------------
  // toString() method
  //---------------------------------------------------------------------------

  /**
   * Return all attributes as String
   * @return Object as String
   */
  @Override
  public String toString()
  {
    StringBuilder buf = new StringBuilder();
    buf.append( " --- toString() of ch.bouquet.jpa.domain.Search --------------------------\n" );

    buf.append( "  search id     <" );
    buf.append( getSearchId() );
    buf.append( ">\n" );
    buf.append( "  subscriber id <" );
    buf.append( getSubscriber().getSubscriberId() );
    buf.append( ">\n" );
    buf.append( "  name          <" );
    buf.append( getName() );
    buf.append( ">\n" );
    buf.append( "  criteria      <" );
    buf.append( getCriteria() );
    buf.append( ">\n" );
    buf.append( "  state         <" );
    buf.append( getState() );
    buf.append( ">\n" );
    buf.append( "  frequency     <" );
    buf.append( getFrequency() );
    buf.append( ">\n" );
    buf.append( "  output format <" );
    buf.append( getOutputFormat() );
    buf.append( ">\n" );
    buf.append( "  start date    <" );
    buf.append( getStartDat() );
    buf.append( ">\n -------------------------------------------------------------------------" );
    return buf.toString();
  }

  /**
   * Return a formatted subscriber as String
   * @return Object as String
   */
  public String toStringFormatted()
  {
    StringBuilder buf = new StringBuilder();
    buf.append( "Der Benutzer <" );
    buf.append( getSubscriber().getSubscriberId() );
    buf.append( "> hat eine Suche gespeichert mit dem Namen <" );
    buf.append( getName() );
    buf.append( ">.\nDiese wird <" );
    buf.append( getFrequency() );
    buf.append( "> durchgefuehrt. Der aktuelle Status ist <" );
    buf.append( getState() );
    buf.append( ">." );
    return buf.toString();
  }

  //-----------------------------------------------------------------------------
  // G E T T E R - / S E T T E R - M E T H O D S
  //-----------------------------------------------------------------------------

  /**
   * Gets the searchId.
   * @return the searchId
   */
  public int getSearchId()
  {
    return this.searchId;
  }

  /**
   * Sets the searchId.
   * @param searchId The searchId to set
   */
  public void setSearchId( int searchId )
  {
    this.searchId = searchId;
  }

  /**
   * Gets the subscriber.
   * @return the subscriber
   */
  public Subscriber getSubscriber()
  {
    return this.subscriber;
  }

  /**
   * Sets the subscriber.
   * @param subscriber The subscriber to set
   */
  public void setSubscriber( Subscriber subscriber )
  {
    this.subscriber = subscriber;
  }

  /**
   * Gets the name.
   * @return the name
   */
  public String getName()
  {
    return this.name;
  }

  /**
   * Sets the name.
   * @param name The name to set
   */
  public void setName( String name )
  {
    this.name = name;
  }

  /**
   * Gets the criteria.
   * @return the criteria
   */
  public String getCriteria()
  {
    return this.criteria;
  }

  /**
   * Sets the criteria.
   * @param criteria The criteria to set
   */
  public void setCriteria( String criteria )
  {
    this.criteria = criteria;
  }

  /**
   * Gets the state.
   * @return the state
   */
  public State getState()
  {
    return this.state;
  }

  /**
   * Sets the state.
   * @param state The state to set
   */
  public void setState( State state )
  {
    this.state = state;
  }

  /**
   * Gets the frequency.
   * @return the frequency
   */
  public Frequency getFrequency()
  {
    return this.frequency;
  }

  /**
   * Sets the frequency.
   * @param frequency The frequency to set
   */
  public void setFrequency( Frequency frequency )
  {
    this.frequency = frequency;
  }

  /**
   * Gets the outputFormat.
   * @return the outputFormat
   */
  public OutputFormat getOutputFormat()
  {
    return this.outputFormat;
  }

  /**
   * Sets the outputFormat.
   * @param outputFormat The outputFormat to set
   */
  public void setOutputFormat( OutputFormat outputFormat )
  {
    this.outputFormat = outputFormat;
  }

  /**
   * Gets the startDat.
   * @return the startDat
   */
  public Date getStartDat()
  {
    return this.startDat;
  }

  /**
   * Sets the startDat.
   * @param startDat The startDat to set
   */
  public void setStartDat( Date startDat )
  {
    this.startDat = startDat;
  }
}